// code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RedBlackTrees {
  public class SimpleNodePainter : IBinaryNodePainter, IDisposable {
    private Graphics surface;

    private Pen edgePen;
    private Pen borderPen;
    private Brush nodeFill;
    private Font font;
    private Brush fontBrush;

    public SimpleNodePainter(Graphics g, float unit) {
      surface = g;
      edgePen = new Pen(Color.Black, 2.0f / unit);
      borderPen = new Pen(Color.Black, 2.0f / unit);
      nodeFill = new SolidBrush(Color.SkyBlue);
      font = new Font("Consolas", 0.5f);
      fontBrush = new SolidBrush(Color.Black);
    }

    public void Dispose() {
      Dispose(true);
      GC.SuppressFinalize(this);
    }
    protected virtual void Dispose(bool disposing) {
      if (disposing) {
        if (edgePen != null) edgePen.Dispose();
        if (borderPen != null) borderPen.Dispose();
        if (nodeFill != null) nodeFill.Dispose();
        if (font != null) font.Dispose();
        if (fontBrush != null) fontBrush.Dispose();
      }
    }

    public void DrawEdge(int childNumber, int childLevel,
                          int parentNumber) {
      int parentY = childLevel - 1;
      surface.DrawLine(edgePen,
                       new PointF(childNumber, childLevel),
                       new PointF(parentNumber, parentY)
                       );
    }
    public void DrawNode(IBinaryNode node, int number, int level) {
      RectangleF nodeRect = new RectangleF(number - 0.5f, level - 0.5f, 1.0f, 1.0f);
      surface.FillEllipse(nodeFill, nodeRect);
      surface.DrawEllipse(borderPen, nodeRect);
      nodeRect.Offset(0.2f, 0.15f);
      surface.DrawString(number.ToString(), font, fontBrush, nodeRect);
    }
    public void DrawEdgedNode(IBinaryNode node,
                              int childNumber, int childLevel,
                              int parentNumber) {
      DrawEdge(childNumber, childLevel, parentNumber);
      DrawNode(node, childNumber, childLevel);
    }
  }
}
